"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reverse = reverse;
exports.sum = sum;
exports.iter = iter;
exports.flatten = flatten;
exports.distinct = distinct;
exports.betweenIncl = betweenIncl;
exports.intersection = intersection;
exports.selectMany = selectMany;
exports.range = range;
exports.aggregate = aggregate;
exports.aggCount = aggCount;
exports.contains = contains;
exports.groupBy = groupBy;
exports.groupByArr = groupByArr;
exports.orderByAscending = orderByAscending;
exports.orderByDescending = orderByDescending;
exports.orderByStrAscending = orderByStrAscending;
exports.orderByStrDescending = orderByStrDescending;
exports.last = last;
exports.first = first;
exports.min = min;
exports.any = any;
exports.all = all;
exports.max = max;
exports.splitNWays = splitNWays;
exports.overlapBetween = overlapBetween;
exports.dedupe = dedupe;
exports.distinctByKey = distinctByKey;
exports.stringDiff = stringDiff;
function reverse(list) {
    var newList = [];
    for (var i = list.length - 1; i >= 0; i--) {
        newList.push(list[i]);
    }
    return newList;
}
function sum(list) {
    return list.reduce(function (acc, v) { return acc + v; }, 0);
}
function iter(iterable) {
    var list = [];
    for (var i = 0; i < iterable.length; i++) {
        list.push(iterable[i]);
    }
    return list;
}
function flatten(input) {
    return input.reduce(function (list, item) {
        return __spreadArray(__spreadArray([], list, true), item, true);
    }, []);
}
function distinct(input, selector) {
    if (!selector)
        selector = function (v) { return v; };
    return input.map(selector).filter(function (value, index, array) { return array.indexOf(value) === index; });
}
function betweenIncl(test, boundA, boundB) {
    var min = Math.min(boundA, boundB);
    var max = Math.max(boundA, boundB);
    return test >= min && test <= max;
}
function intersection(a, b) {
    return a.filter(function (el) { return b.indexOf(el) !== -1; });
}
function selectMany(input, selector) {
    return flatten(input.map(selector));
}
function range(a, b) {
    if (b === void 0) { b = null; }
    var start = a;
    var end;
    if (b === null) {
        start = 0;
        end = a;
    }
    else {
        end = b;
    }
    var out = [];
    for (var i = start; i < end; i++) {
        out.push(i);
    }
    return out;
}
function aggregate(list, keySelector, aggregate) {
    return list.reduce(function (acc, value) {
        var key = keySelector(value);
        var oldValue = acc[key] || 0;
        acc[key] = aggregate(value, oldValue);
        return acc;
    }, {});
}
function aggCount(list, keySelector) {
    return aggregate(list, keySelector, function (current, acc) { return 1 + acc; });
}
function contains(arr, search) {
    return arr.indexOf(search) !== -1;
}
function groupBy(list, keySelector) {
    return list.reduce(function (accumulator, value) {
        var _a;
        var _b;
        var key = keySelector(value);
        var arr = (_b = accumulator[key]) !== null && _b !== void 0 ? _b : [];
        return Object.assign(accumulator, (_a = {},
            _a[key] = __spreadArray(__spreadArray([], arr, true), [value], false),
            _a));
    }, {});
}
function groupByArr(list, keySelector, compareKeyByReference) {
    if (compareKeyByReference === void 0) { compareKeyByReference = false; }
    var keys = [];
    return list.reduce(function (accumulator, value) {
        var keyValue = keySelector(value);
        var lookupKey = keyValue;
        if (!compareKeyByReference)
            lookupKey = JSON.stringify(keyValue);
        var index = keys.indexOf(lookupKey);
        if (index === -1) {
            accumulator.push({ key: keyValue, values: [value] });
            keys.push(lookupKey);
            return accumulator;
        }
        accumulator[index].values.push(value);
        return accumulator;
    }, []);
}
function orderByAscending(list, selector) {
    list.sort(function (a, b) { return selector(a) - selector(b); });
}
function orderByDescending(list, selector) {
    list.sort(function (a, b) { return selector(b) - selector(a); });
}
function orderByStrAscending(list, selector) {
    list.sort(function (a, b) {
        var aVal = selector(a);
        var bVal = selector(b);
        if (aVal < bVal)
            return -1;
        if (aVal > bVal)
            return 1;
        return 0;
    });
}
function orderByStrDescending(list, selector) {
    list.sort(function (a, b) {
        var aVal = selector(a);
        var bVal = selector(b);
        if (aVal < bVal)
            return 1;
        if (aVal > bVal)
            return -1;
        return 0;
    });
}
function last(items, selector) {
    for (var i = items.length - 1; i >= 0; i--) {
        if (selector(items[i]))
            return items[i];
    }
    return null;
}
function first(items, selector) {
    for (var i = 0; i < items.length; i++) {
        if (selector(items[i]))
            return items[i];
    }
    return null;
}
function min(items, selector) {
    if (items.length === 0)
        throw new Error('no items in list');
    var minIndex = 0;
    var minValue = Infinity;
    items
        .map(function (item) { return selector(item); })
        .map(function (value, i) {
        if (value < minValue) {
            minIndex = i;
            minValue = value;
        }
        return null;
    });
    return items[minIndex];
}
function any(items, selector) {
    for (var i = 0; i < items.length; i++) {
        if (selector(items[i]))
            return true;
    }
    return false;
}
function all(items, selector) {
    return items.filter(selector).length === items.length;
}
function max(items, selector) {
    if (items.length === 0)
        throw new Error('no items in list');
    var maxIndex = 0;
    var maxValue = -Infinity;
    items
        .map(function (item) { return selector(item); })
        .map(function (value, i) {
        if (value > maxValue) {
            maxIndex = i;
            maxValue = value;
        }
        return null;
    });
    return items[maxIndex];
}
function splitNWays(elements, n) {
    if (elements.length < n) {
        throw new Error("can't split a list of " + elements.length + ' elements ' + n + ' ways');
    }
    var position = 0;
    return range(n).map(function (index) {
        var howMany = Math.floor((elements.length - position) / (n - index));
        var result = elements.slice(position, position + howMany);
        position = position + howMany;
        return result;
    });
}
function overlapBetween(a, b) {
    return a.filter(function (v) { return contains(b, v); });
}
function dedupe(list) {
    return list.filter(function (item, index) {
        for (var i = index + 1; i < list.length; i++) {
            if (list[i] === item)
                return false;
        }
        return true;
    });
}
function distinctByKey(value, keySelector) {
    var keys = [];
    var out = [];
    for (var _i = 0, value_1 = value; _i < value_1.length; _i++) {
        var v = value_1[_i];
        var key = keySelector(v);
        if (keys.indexOf(key) !== -1)
            continue;
        keys.push(key);
        out.push(v);
    }
    return out;
}
function stringDiff(a, b) {
    if (a === b)
        return 0;
    if (a > b)
        return -1;
    return 1;
}
