import { ParsedUrlQueryInput } from 'querystring';
type PreflightInput = {
    path: string;
    params: RequestInit;
};
export type XhrHooks = {
    input?: (input: PreflightInput) => PreflightInput;
    preSend?: (xhr: XMLHttpRequest) => void;
};
type PreflightMiddleware = (input: PreflightInput) => PreflightInput;
export type ProgressCallback = (e: ProgressEvent<XMLHttpRequestEventTarget>) => any;
export declare class Fetcher {
    defaultHeaders: Headers;
    on401: (retry: () => Promise<any>) => Promise<any>;
    preflight: PreflightMiddleware[];
    constructor(options: {
        on401: (retry: () => Promise<any>) => Promise<any>;
        apiVersion?: string;
    });
    updatePath(path: string): string;
    download(method: 'GET', path: string, urlParameters?: ParsedUrlQueryInput): void;
    getHeaders(add?: {
        [k: string]: string;
    }): Headers;
    postFormData<T>(path: string, body: {
        [k: string]: string | Blob;
    }, isRetry?: boolean): Promise<T>;
    fetchWithProgress<T>(method: 'POST' | 'PUT', path: string, body: BodyInit, onProgress: ProgressCallback, hooks?: XhrHooks): Promise<T>;
    postFormDataWithProgress<T>(path: string, body: {
        [k: string]: string | Blob;
    }, onProgress: ProgressCallback, hooks?: XhrHooks | null, isRetry?: boolean): Promise<T>;
    xhr(path: string, params: RequestInit, onProgress: (e: ProgressEvent<XMLHttpRequestEventTarget>) => any, hooks?: XhrHooks | null): Promise<XMLHttpRequest>;
    fetch(path: string, params: RequestInit): Promise<Response>;
    postForAuth<T>(path: string, body: any): Promise<T>;
    post<T>(path: string, body: any, isRetry?: boolean): Promise<T>;
    put<T>(path: string, body: any, retry?: boolean): Promise<T>;
    get<T>(path: string, urlParameters?: ParsedUrlQueryInput, isRetry?: boolean): Promise<T>;
    handleXhrResponse<T>(xhr: XMLHttpRequest, retry?: () => Promise<T>, isRetry?: boolean): Promise<T>;
    isOkStatus(value: number): boolean;
    handleFetchResponse<T>(result: Response, retry?: () => Promise<T>, isRetry?: boolean): Promise<T>;
}
export interface ErrResponse {
    error: string;
}
export {};
