"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useWebSocket = useWebSocket;
var react_1 = require("react");
var BrowserWindowId_1 = require("./BrowserWindowId");
var Auth_1 = require("./Auth");
function useWebSocket(path, onMessage, onMessageDeps, props) {
    var propsRef = (0, react_1.useRef)(props);
    propsRef.current = props;
    var _a = (0, react_1.useState)(false), refresh = _a[0], setRefresh = _a[1];
    var _b = (0, react_1.useState)(), socket = _b[0], setSocket = _b[1];
    var _c = (0, react_1.useState)(false), ready = _c[0], setReady = _c[1];
    var auth = (0, Auth_1.useAuthenticated)();
    var isAuthenticated = auth.authenticated;
    (0, react_1.useEffect)(function () {
        var protocol = window.location.protocol === 'http:' ? 'ws:' : 'wss:';
        var pathParts = path.split('?');
        if (pathParts.length === 1) {
            pathParts.push('');
        }
        var query = new URLSearchParams(pathParts[1]);
        query.set('browser-window-id', BrowserWindowId_1.browserWindowId);
        pathParts[1] = query.toString();
        path = pathParts.join('?');
        var s = new WebSocket(protocol + '//' + window.location.host + path);
        setSocket(s);
        return function () {
            try {
                if (s.readyState === WebSocket.CLOSED || s.readyState === WebSocket.CLOSING) {
                    return;
                }
                s.close();
            }
            catch (e) { }
        };
    }, [refresh, path, isAuthenticated]);
    (0, react_1.useEffect)(function () {
        if (!socket)
            return;
        var callback = function (e) {
            var _a;
            setReady(true);
            if (!((_a = propsRef.current) === null || _a === void 0 ? void 0 : _a.onOpen))
                return;
            propsRef.current.onOpen(socket, e);
        };
        socket.addEventListener('open', callback);
        return function () {
            socket.removeEventListener('open', callback);
        };
    }, [socket, propsRef]);
    var lastRefreshRef = (0, react_1.useRef)([]);
    (0, react_1.useEffect)(function () {
        if (!socket)
            return;
        var timeout = 0;
        socket.addEventListener('close', function (e) {
            var _a;
            setReady(false);
            if ((_a = propsRef.current) === null || _a === void 0 ? void 0 : _a.onClose)
                propsRef.current.onClose(socket, e);
            var retryIn = 1000;
            if (e.code === 1006) {
                retryIn = 10 * 1000;
                var now_1 = new Date().getTime();
                var last5min = lastRefreshRef.current.filter(function (r) { return r > now_1 - 5 * 60 * 1000; });
                if (last5min.length > 5) {
                    retryIn = 30 * 1000;
                }
            }
            timeout = setTimeout(function () {
                lastRefreshRef.current.push(new Date().getTime());
                setRefresh(function (old) { return !old; });
            }, retryIn);
        });
        return function () {
            if (timeout)
                clearTimeout(timeout);
        };
    }, [socket, propsRef]);
    (0, react_1.useEffect)(function () {
        if (!socket)
            return;
        var callback = function (e) { return onMessage(e.data); };
        socket.addEventListener('message', callback);
        return function () {
            socket.removeEventListener('message', callback);
        };
    }, [socket, onMessage]);
    // only return socket once it's ready to make things more straightforward for people
    // doing stuff like `useEffect(() => {}, [socket, projectId]);`
    return ready ? socket : undefined;
}
